<?php

namespace Drupal\content_snapshot\Normalizer;

use Drupal\content_snapshot\Encoder\YamlEncoderInterface;
use Drupal\serialization\Normalizer\FieldItemNormalizer;
use Drupal\pathauto\PathautoState;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\path\Plugin\Field\FieldType\PathItem;
use Drupal\pathauto\PathautoGeneratorInterface;

/**
 * Class for PathFieldItemNormalizer.
 */
class PathFieldItemNormalizer extends FieldItemNormalizer {

  use ContentSnapshotNormalizerContextTrait;

  /**
   * Format.
   *
   * @var string
   */
  protected $format = YamlEncoderInterface::FORMAT;

  /**
   * Supported interface or class.
   *
   * @var string
   */
  protected $supportedInterfaceOrClass = PathItem::class;

  /**
   * Path auto generator.
   *
   * @var \Drupal\pathauto\PathautoGeneratorInterface|null
   */
  private $pathautoGenerator;

  /**
   * PathItemNormalizer constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *
   *   Entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Set Pathauto Generator.
   *
   * @param \Drupal\pathauto\PathautoGeneratorInterface $pathautoGenerator
   *   Path auto generator.
   */
  public function setPathautoGenerator(PathautoGeneratorInterface $pathautoGenerator) {
    $this->pathautoGenerator = $pathautoGenerator;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\Core\Field\FieldItemInterface $field
   *   The Field Item Interface.
   * @param string $format
   *   The format.
   * @param array $context
   *   The context.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function normalize($field, $format = NULL, array $context = []):// phpcs:ignore Drupal.Commenting.FunctionComment.TypeHintMissing,Drupal.Files.LineLength.TooLong
  array|\ArrayObject|bool|float|int|NULL|string {

    $this->validateContext($context);
    $attributes = parent::normalize($field, $format, $context);
    /** @var \Drupal\Core\Entity\FieldableEntityInterface $entity */
    $entity = $field->getParent()->getParent()->getValue();

    if ($this->pathautoGenerator && array_key_exists('pathauto', $field->getProperties(TRUE))) {
      $pathauto = $field->get('pathauto');
      if (
        $pathauto instanceof PathautoState &&
        $this->pathautoGenerator->getPatternByEntity($entity)
      ) {
        $attributes['pathauto'] = $pathauto->getValue();
      }
    }

    return $attributes;
  }

  /**
   * {@inheritDoc}
   */
  public function denormalize($data, $class, $format = NULL, array $context = []): mixed {

    $this->validateContext($context);

    /** @var \Drupal\Core\Field\FieldItemInterface $targetInstance */
    $targetInstance = $context['target_instance'];

    /** @var \Drupal\Core\Entity\FieldableEntityInterface $entity */
    $entity = $targetInstance->getParent()->getParent()->getValue();

    // We need to make sure that pathauto is disabled during the whole
    // importing or we are risking creating duplicated path aliases.
    if ($this->pathautoGenerator && array_key_exists('pathauto', $targetInstance->getProperties(TRUE))) {
      /** @var \Drupal\pathauto\PathautoState $pathauto */
      $pathauto = $targetInstance->get('pathauto');

      if ($pathauto instanceof PathautoState) {
        if (
          isset($data['pathauto']) &&
          $this->pathautoGenerator->getPatternByEntity($entity)
        ) {
          $pathauto->setValue(0, FALSE);
        }
      }
    }

    return NULL;
  }

}
