<?php

namespace Drupal\content_snapshot\Snapshot;

/**
 * Class for SnapshotItem.
 */
class SnapshotItem implements SnapshotItemInterface {

  /**
   * Id of snapshot item's type.
   */
  protected const TYPE = 'generic_content';

  /**
   * Key under which content's metadata should be stored in snapshot array.
   */
  private const METADATA_KEY = '_content_snapshot_metadata';

  /**
   * Id.
   */
  private string $id;

  /**
   * Data.
   */
  private array $data;

  /**
   * Uuid.
   */
  private string $getUuid;

  /**
   * Priority.
   */
  private int $priority = 0;

  /**
   * Excluded.
   */
  private bool $excluded = FALSE;

  /**
   * SnapshotItem constructor.
   *
   * @param string $id
   *   Id of item.
   * @param string $getUuid
   *   Snapshot uuid.
   * @param array $data
   *   Snapshot data.
   */
  public function __construct(string $id, string $getUuid, array $data) {
    $this->id = $id;
    $this->getUuid = $getUuid;
    $this->data = $data;
  }

  /**
   * {@inheritDoc}
   */
  public function getId(): string {
    return $this->id;
  }

  /**
   * {@inheritDoc}
   */
  public function getUuid(): string {
    return $this->getUuid;
  }

  /**
   * {@inheritDoc}
   */
  public function getPath(): ?string {
    return NULL;
  }

  /**
   * {@inheritDoc}
   */
  public function getNormalizedContent(): array {
    return $this->data;
  }

  /**
   * {@inheritDoc}
   */
  public function setPriority(int $priority) {
    $this->priority = $priority;
  }

  /**
   * {@inheritDoc}
   */
  public function getPriority(): int {
    return $this->priority;
  }

  /**
   * {@inheritDoc}
   */
  public function isExcluded(): bool {
    return $this->excluded;
  }

  /**
   * {@inheritDoc}
   */
  public function setExcluded(bool $bool): void {
    $this->excluded = $bool;
  }

  /**
   * {@inheritDoc}
   */
  public function toArray(): array {
    $data = $this->data;
    $data[self::METADATA_KEY] = [
      'id' => $this->id,
      'uuid' => $this->getUuid,
      'type' => static::TYPE,
      'additional' => $this->getAdditionalMetadata(),
    ];

    return $data;
  }

  /**
   * Function form array.
   *
   * @param array $array
   *   Form array.
   *
   * @return self|null
   *   SnapshotItem created from array.
   */
  public static function fromArray(array $array): ?SnapshotItemInterface {

    $type = static::extractTypeFromArray($array);

    if ($type !== static::TYPE) {
      return NULL;
    }

    $metadataArray = $array[self::METADATA_KEY];
    unset($array[self::METADATA_KEY]);

    return new static($metadataArray['id'], $metadataArray['uuid'], $array);
  }

  /**
   * Get additional metadata.
   *
   * @return array
   *   Array with additional metadata.
   */
  protected function getAdditionalMetadata(): array {
    return [];
  }

  /**
   * Extracts snapshot item's id from an array.
   *
   * @param array $array
   *   Snapshot item as an array.
   *
   * @return string
   *   Id.
   */
  protected static function extractIdFromArray(array $array): string {
    return $array[self::METADATA_KEY]['id'];
  }

  /**
   * Extracts snapshot item's type from an array.
   *
   * @param array $array
   *   Snapshot item as an array.
   *
   * @return string
   *   Snapshot item's type.
   */
  protected static function extractTypeFromArray(array $array): string {
    return $array[self::METADATA_KEY]['type'];
  }

  /**
   * Checks if snapshot item has UUID.
   *
   * @param array $array
   *   Snapshot item as an array.
   *
   * @return bool
   *   Does snapshot item have UUID?
   */
  protected static function extractUuidFromArray(array $array): bool {
    return (bool) $array[self::METADATA_KEY]['uuid'];
  }

  /**
   * Extracts additional metadata from an array.
   *
   * @param array $array
   *   Snapshot item as an array.
   *
   * @return array
   *   Additional metadata.
   */
  protected static function extractAdditionalMetadataFromArray(array $array): array {
    return $array[self::METADATA_KEY]['additional'];
  }

  /**
   * Removes metadata from array.
   *
   * @param array $array
   *   Snapshot item as an array.
   *
   * @return array
   *   Array with metadata removed.
   */
  protected static function removeMetadataFromArray(array $array): array {
    unset($array[self::METADATA_KEY]);

    return $array;
  }

}
