<?php

namespace Drupal\Tests\content_snapshot\Functional\modules\pathauto;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\pathauto\Entity\PathautoPattern;
use Drupal\pathauto\PathautoState;
use Drupal\Tests\content_snapshot\Functional\AbstractFunctional;

/**
 * Class PathautoTest.
 *
 * This class tests pathauto module handling.
 *
 * @coversDefaultClass \Drupal\content_snapshot\EventSubscriber\PathautoRestorerEventSubscriber
 *
 * @group content_snapshot
 */
class PathautoTest extends AbstractFunctional {

  /**
   * UUID generator.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  private UuidInterface $uuid;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'pathauto',
    'content_snapshot',
  ];

  /**
   * Node storage.
   *
   * @var \Drupal\node\NodeStorageInterface
   */
  protected $nodeStorage;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->uuid = \Drupal::service('uuid');

    $this->setUpSnapshotPath();

    $this->nodeStorage = $this->entityTypeManager->getStorage('node');

    $this->createContentType([
      'type' => 'page',
      'name' => 'Page type',
    ]);

    $this->createContentType([
      'type' => 'article',
      'name' => 'Article type',
    ]);

  }

  /**
   * Tests if paragraphs export import export produces consistent snapshots.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testPathautoStatePersistence() {

    $this->createPattern('title', '/[node:title]', ['page']);

    $node1 = $this->createNode([
      'type' => 'page',
      'path' => [
        'pathauto' => PathautoState::CREATE,
      ],
    ]);
    $node1->save();

    $node2 = $this->createNode([
      'type' => 'page',
      'path' => [
        'pathauto' => PathautoState::SKIP,
      ],
    ]);
    $node2->save();

    $this->assertSnapshotConsistency();

    $node1Imported = $this->nodeStorage->load($node1->id());
    $node2Imported = $this->nodeStorage->load($node2->id());

    $this->assertEquals($node1Imported->get('path')->pathauto, PathautoState::CREATE);
    $this->assertEquals($node2Imported->get('path')->pathauto, PathautoState::SKIP);
  }

  /**
   * Creates a pathauto pattern for given content types.
   *
   * @param string $id
   *   Pattern's id.
   * @param string $pattern
   *   Pattern string.
   * @param string[] $bundles
   *   Array of bundle ids.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function createPattern(string $id, string $pattern, array $bundles): void {
    $bundlesConfig = [];
    foreach ($bundles as $bundle) {
      $bundlesConfig[$bundle] = $bundle;
    }

    $uuid = $this->uuid->generate();
    $pattern = PathautoPattern::create([
      'id' => $id,
      'label' => $id,
      'pattern' => $pattern,
      'status' => TRUE,
      'type' => 'canonical_entities:node',
      'selection_logic' => 'and',
      'selection_criteria' => [
        $uuid => [
          'id' => 'entity_bundle:node',
          'negate' => FALSE,
          'uuid' => $uuid,
          'context_mapping' => [
            'node' => 'node',
          ],
          'bundles' => $bundlesConfig,
        ],
      ],
    ]);

    $pattern->save();
  }

}
